/* 
 * Copyright (c) 2000-2001, catchy.net. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *      - Redistributions of source code must retain the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer.
 * 
 *      - Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 * 
 *      - All advertising materials mentioning features or use of this
 *      software must display the following acknowledgement: This
 *      product includes software developed by catchy.net and its
 *      contributors.
 * 
 *      - Neither name of catchy.net nor the names of its contributors
 *      may be used to endorse or promote products derived from this
 *      software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY CATCHY.NET AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL CATCHY.NET OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * 
 * $Id: FontSupport.cpp,v 1.2 2001/03/22 01:46:22 pascal Exp $ 
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <X11/Xlib.h>

#include "FontSupport.h"

extern "C" {

/**
 * Retrieves a list of X LFD's. See XListFonts(3X11).
 */
void getFontList(char ***fontlist, int *n_fonts) {

  /* Get a list of all the fonts from the X server. */
  Display *dpy = XOpenDisplay(NULL);
  *fontlist = XListFonts(dpy, "-*-*-*-*-*-*-*-*-*-*-*-*-iso8859-*", 
                         3000, 
                         n_fonts);
  XCloseDisplay(dpy);
  
}

/**
 * Releases a list of X LFD's.
 */
void freeFontList(char **fontlist) {
  
  XFreeFontNames(fontlist);

}

/**
 * Given an LFD, creates a FontFamily.
 */
FontFamily *createFontFamily(const char *lfd) {
  char *s, *s2, *fd;
  FontFamily *fontFamily = NULL;

  fd = (char *) malloc(strlen(lfd) + 1);

  if(fd) {

    strcpy(fd, lfd);

    fontFamily = (FontFamily *) malloc(sizeof(FontFamily));

    if(fontFamily) {

      s = strchr(fd + 1, '-');

      fontFamily->foundry = (char *) malloc(s - fd);

      if(fontFamily->foundry) {

        memcpy(fontFamily->foundry, fd + 1, s - (fd + 1));
        fontFamily->foundry[s - (fd + 1)] = '\0';

        s2 = strchr(s + 1, '-');

        fontFamily->name = (char *) malloc(s2 - s);

        if(fontFamily->name) {
        
          memcpy(fontFamily->name, s + 1, s2 - (s + 1));
          fontFamily->name[s2 - (s + 1)] = '\0';

          fontFamily->friendlyName = 
            (char *) malloc(strlen(fontFamily->name) + 
                            strlen(fontFamily->foundry) + 5);
        
          if(fontFamily->friendlyName) {

            sprintf(fontFamily->friendlyName, "%s (%s)", 
                    fontFamily->name, 
                    fontFamily->foundry);
            fontFamily->LFD = fd;

          } else {

            /* Failed to allocate memory for friendly name. */
            free(fontFamily->name);
            free(fontFamily->foundry);
            free(fontFamily);
            fontFamily = NULL;
          
          }

        } else {

          /* Failed to allocate memory for name. */
          free(fontFamily->foundry);
          free(fontFamily);
          fontFamily = NULL;

        }

      } else {

        /* Failed to allocate memory for foundry. */
        free(fontFamily);
        fontFamily = NULL;

      }

    }

  }

  return fontFamily;

}

/**
 * Clones a FontFamily and returns the copy.
 */
FontFamily *copyFontFamily(const FontFamily *original) {
  FontFamily *copy;

  copy = (FontFamily *) malloc(sizeof(FontFamily));

  if(copy) {

    copy->name = (char *) malloc(strlen(original->name) + 1);
    
    if(copy->name) {

      strcpy(copy->name, original->name);
      
      copy->foundry = (char *) malloc(strlen(original->foundry) + 1);
      
      if(copy->foundry) {

        strcpy(copy->foundry, original->foundry);

        copy->friendlyName = (char *) malloc(strlen(original->friendlyName) + 1);
        
        if(copy->friendlyName) {

          strcpy(copy->friendlyName, original->friendlyName);

          copy->LFD = (char *) malloc(strlen(original->LFD) + 1);

          if(copy->LFD) {

            strcpy(copy->LFD, original->LFD);

          } else {

            free(copy->friendlyName);
            free(copy->foundry);
            free(copy->name);
            free(copy);
            copy = NULL;

          }

        } else {

          free(copy->foundry);
          free(copy->name);
          free(copy);
          copy = NULL;

        }

      } else {

        free(copy->name);
        free(copy);
        copy = NULL;

      }

    } else {

      free(copy);
      copy = NULL;

    }

  }

  return copy;

}

/**
 * Destroys a FontFamily.
 */
void destroyFontFamily(FontFamily *fontFamily) {

  if(fontFamily) {

    if(fontFamily->name)
      free(fontFamily->name);

    if(fontFamily->foundry)
      free(fontFamily->foundry);

    if(fontFamily->friendlyName)
      free(fontFamily->friendlyName);

    free(fontFamily->LFD);

    free(fontFamily);

  }

}

} /* extern "C" */

