/* 
 * Copyright (c) 2000-2001, catchy.net. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      - Redistributions of source code must retain the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer.
 * 
 *      - Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 * 
 *      - All advertising materials mentioning features or use of this
 *      software must display the following acknowledgement: This
 *      product includes software developed by catchy.net and its
 *      contributors.
 * 
 *      - Neither name of catchy.net nor the names of its contributors
 *      may be used to endorse or promote products derived from this
 *      software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY CATCHY.NET AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL CATCHY.NET OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * 
 * $Id: GtkOptionMenuPeer.cpp,v 1.1 2001/03/11 08:43:40 pascal Exp $ */

#include <stdio.h>
#include <gnome.h>

#include "../../lib/String.h"
#include "../../lib/PointerList.h"
#include "../../renderer/GUIPeer.h"
#include "GtkOptionMenuPeer.h"

/**
 * Creates a GtkOption menu and fills it with the String objects
 * a given by optionList.
 * @param canvas the canvas to attach to.
 * @param optionList the list of options that should appear in the widget.
 */
GtkOptionMenuPeer::GtkOptionMenuPeer(GtkWidget *canvas, PointerList *optionList) {
  unsigned int i;
  void **l = optionList->getArray();

  /* Create an option menu. */
  pointer = gtk_option_menu_new();

  /* Create a menu from the list of options. */
  optionMenu = GTK_MENU(gtk_menu_new());

  GtkWidget *item;

  /* Add the options to the menu. */
  for(i = 0; i < optionList->getSize(); i++) {
    item = gtk_menu_item_new_with_label(((String *) l[i])->getPtr());
    gtk_widget_show(item);
    gtk_menu_append(optionMenu, item);
  }

  gtk_menu_set_active(optionMenu, 0);

  /* Add the menu to the option menu and show the option menu. */
  gtk_option_menu_set_menu(GTK_OPTION_MENU(pointer), GTK_WIDGET(optionMenu));
  gtk_widget_show(GTK_WIDGET(optionMenu));

  /* Put the select box somewhere in a hopefully invisible place on
     the canvas. */
  gtk_layout_put(GTK_LAYOUT(canvas), GTK_WIDGET(pointer), -1000, -1000);

  /* Show the widget and process events to get it's size. */
  gtk_widget_show(GTK_WIDGET(pointer));
  while(gtk_events_pending())
    gtk_main_iteration_do(0);

  /* Gotcha. */
  width = GTK_WIDGET(pointer)->allocation.width;
  height = GTK_WIDGET(pointer)->allocation.height;

}

GtkOptionMenuPeer::~GtkOptionMenuPeer() {

  gtk_widget_destroy(GTK_WIDGET(pointer));

}

