/* 
 * Copyright (c) 2000-2001, catchy.net. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *      - Redistributions of source code must retain the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer.
 * 
 *      - Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 * 
 *      - All advertising materials mentioning features or use of this
 *      software must display the following acknowledgement: This
 *      product includes software developed by catchy.net and its
 *      contributors.
 * 
 *      - Neither name of catchy.net nor the names of its contributors
 *      may be used to endorse or promote products derived from this
 *      software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY CATCHY.NET AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL CATCHY.NET OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * 
 * $Id: Mobilizer.h,v 1.3 2001/03/21 16:03:28 pascal Exp $ 
 */

#ifndef __MOBILIZER_H__
#define __MOBILIZER_H__

#include <gnome.h>

#include "../../renderer/WMLWindow.h"
#include "GNOMEInterface.h"
#include "FontSupport.h"

typedef struct {
  /** Hand mouse pointer (when hovering over links). */
  GdkCursor *hand;
  /** Standard mouse pointer. */
  GdkCursor *standard;
} MouseCursors;

typedef struct {
  /** Font list as returned by the X server. */
  char **elements;
  /** Size of the font list. */
  int size;
} XFontList;

typedef struct {
  /** The color to use when drawing links. */
  GdkColor link;
  /** The color to use when drawing active links. */
  GdkColor activeLink;
  /** The foreground color to use when drawing text or bitmaps. */
  GdkColor foreground;
  /** The background color to use when drawing text or bitmaps. */
  GdkColor background;
  /** Temporary placeholders for the above colors during the time when
      a user has selected a new color but not chosen to apply. */
  GdkColor _link;
  GdkColor _activeLink;
  GdkColor _foreground;
  GdkColor _background;
} CurrentColors;

typedef struct {
  /** The size of the current font in pixels. */
  int size;
  /** The font info specifying the font family. */
  FontFamily *family;
} CurrentFont;

typedef struct {
  /** Pointers to the color pickers for text color etc., necessary to
      set the right colors. */
  GtkWidget *background;
  GtkWidget *foreground;
  GtkWidget *link;
  GtkWidget *activeLink;
} ColorSelectionInterface;

typedef struct {
  /** Pointers to the history buttons, necessary to hide / show
      them. */
  GtkWidget *back;
  GtkWidget *forward;
} HistoryInterface;

typedef struct {
  /** Pointer to the OptionMenu which displays the font list,
      necessary to attach the Menu. */
  GtkWidget *families;
  /** Pointer to the GtkEditable which contains the font size,
      necessary to set / retrieve the value. */
  GtkWidget *size;
  /** Pointer to the Menu inside the OptionMenu, necessary to add
      items to the Menu. */
  GtkMenu *familiesMenu;
  /** A Glib singly linked list which keeps track of the FontInfo
      structures. */
  GSList *fontInfoList;
  /** The font list as returned by the X server. */
  XFontList fontList;
} FontSelectionInterface;

/**
 * A BrowserInterface encapsulates all the information relevant to a
 * browser window.
 */
typedef struct {
  /** Pointer to the WMLWindow which renders the content. */
  WMLWindow *ww;
  /** The drawing interface that is used by the WMLWindow. */
  GNOMEInterface *wdi;
  /** The graphics context for drawing operations. */
  GdkGC *gc;
  /** The color map to use. */
  GdkColormap *colorMap;
  /** Pointer to a scrolled window which contains a GtkLayout. */
  GtkWidget *scrolled_window;
  /** Pointer to a GtkLayout which displays the pages. */
  GtkWidget *contentarea;
  /** Pointer to the status bar. */
  GtkWidget *statusbar;
  /** Pointer to the location bar. */
  GtkWidget *location;
  /** Pointer to the home button. */
  GtkWidget *home;
  /** Pointer to the reload button. */
  GtkWidget *reload;
  /** Pointer to the open button. */
  GtkWidget *open;
  /** The scroll bar policies, to turn the scrollbars on and off. */
  GtkPolicyType horizontalScrollBarPolicy, verticalScrollBarPolicy;
  /** The adjustments for the scrollbars. */
  GtkAdjustment *horizontalAdjustment, *verticalAdjustment;
  /** Tracks the width of the (GtkLayout) content area. */
  int width;
  /** Tracks the height of the (GtkLayout) content area. */
  int height;
  /** Tracks the width of the content that is being rendered. */
  int layout_width;
  /** Tracks the height of the content that is being rendered. */
  int layout_height;
  /** The history interface (back, forward, ...). */
  HistoryInterface historyInterface;
  /** Mouse cursors. */
  MouseCursors mouseCursors;
} BrowserInterface;

typedef struct {
  /** Pointer to the preferences dialog, to enable 'Apply' button. */
  GtkWidget *preferences;
  /** Pointer to the font selection interface. */
  FontSelectionInterface fontSelectionInterface;
  /** Pointer to the color selection interface. */
  ColorSelectionInterface colorSelectionInterface;
} PreferencesInterface;

typedef struct {
  /** The file containing our Glade interface. */
  char *gladeInterface;
  /** The current colors to use when rendering. */
  CurrentColors currentColors;
  /** The current font to use when rendering. */
  CurrentFont currentFont;
  /** The browser interface (colors, fonts, widgets, ...). */
  BrowserInterface browserInterface;
  /** The preferences interface (colors, fonts, ...). */
  PreferencesInterface preferencesInterface;
} MobilizerGlobals;

/** Authors, for use in About dialog box. */
static const gchar *authors[] = {
  "Wouter van der Kamp <wouter@catchy.net>",
  "Pascal Haakmat <a.haakmat@chello.nl>",
  NULL
};

void setupGlobals(int argc, char *argv[]);
void setupBrowserInterface(BrowserInterface *bi);

URI *filenameToURI(char *filename);
void loadSource(BrowserInterface *bi, URI *u);

#endif /* __MOBILIZER_H__ */
