/* 
 * Copyright (c) 2000-2001, catchy.net. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      - Redistributions of source code must retain the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer.
 * 
 *      - Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 * 
 *      - All advertising materials mentioning features or use of this
 *      software must display the following acknowledgement: This
 *      product includes software developed by catchy.net and its
 *      contributors.
 * 
 *      - Neither name of catchy.net nor the names of its contributors
 *      may be used to endorse or promote products derived from this
 *      software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY CATCHY.NET AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL CATCHY.NET OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * 
 * $Id: WindowsInterface.h,v 1.2 2001/03/14 13:35:48 wouter Exp $ 
 */
 
/* WindowsInterface.h: interface for the WindowsInterface class.
*/

#ifndef __WINDOWSINTERFACE_H__
#define __WINDOWSINTERFACE_H__

#include <windows.h>

#include "../../renderer/WindowDrawingInterface.h"

/**
 * This class is a class wich WMLWindow uses to display content on a 
 * Microsoft Windows operating system. 
 * For example:
 * \code
 *   WindowsInterface *wi = new WindowsInterface();
 *   WMLWindow *ww = new WMLWindow((WindowDrawingInterface*) wi);
 *   ww->drawContentWindow();
 * \endcode
 */
class WindowsInterface : public WindowDrawingInterface {

public:

  WindowsInterface(HWND hwnd, HINSTANCE hInstance, HFONT hFont);
  ~WindowsInterface();
  void setPreferences(MobilizerPreferences *mp);

  void startDrawing(int width, int height);
  void stopDrawing();

  int getStringHeight(String *s, int style);
  int getStringWidth(String *s, int style);
  int getPixelsBelowBaseLine(int style);
  void drawText(String* text, int x, int y, int style);

  GUIPeer *createInputElement();
  void drawInputElement(WMLGUIElement *wie);

  GUIPeer *createSelectElement(PointerList *optionList);
  void drawSelectElement(WMLGUIElement *wse);

  GUIPeer *createImage(WBMP* wbmp);
  void drawImage(WMLGUIElement *image);

private:

  void selectFont(int style);
  static void drawBitmap(HDC hdc, HBITMAP hBitmap, int xStart, int yStart);

  /* member data */
  MobilizerPreferences *prefs;  /**< the pointer to the preferences */
  HWND windowHandle;            /**< the handle to the content window */
  HINSTANCE instance;           /**< the handle to the instance of this program */
  HFONT font;                   /**< the handle to the used font */
  HDC deviceContext;            /**< the handle to the device context
                                     assosiated with the content window */
  HDC memoryBlock;              /**< the handle to the memory block to which this object
                                     writes before writing to the device context */
  PAINTSTRUCT paintStruct;      /**< the paint struct that is given by BeginPaint and
                                     must be given back to EndPaint */
  int currentStyle;             /**< the style that is currently used for displaying
                                     text, used for performance reasons */
  int windowWidth;              /**< the width of the window, when null, 
                                     actual drawing won't take place*/
  int windowHeight;             /**< the height of the window, when null, 
                                     actual drawing won't take place*/  

};


#endif

