/* 
 * Copyright (c) 2000-2001, catchy.net. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      - Redistributions of source code must retain the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer.
 * 
 *      - Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 * 
 *      - All advertising materials mentioning features or use of this
 *      software must display the following acknowledgement: This
 *      product includes software developed by catchy.net and its
 *      contributors.
 * 
 *      - Neither name of catchy.net nor the names of its contributors
 *      may be used to endorse or promote products derived from this
 *      software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY CATCHY.NET AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL CATCHY.NET OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * 
 * $Id: AbstractSocket.h,v 1.3 2001/03/07 11:47:39 pascal Exp $ 
 */

#ifndef __ABSTRACTSOCKET_H__
#define __ABSTRACTSOCKET_H__

#include <sys/types.h>

#include "String.h"

/**
 * A socket can be used for reading and writing from and to the 
 * network.
 */
class AbstractSocket {
 public:
  AbstractSocket();
  virtual ~AbstractSocket();

  void reset();
  bool isEOF();
  char getChar();
  String *readLine();
  size_t readFully(void *buffer, size_t count);
  void writeString(const String *s);

 private:
  /**
   * Internal String object.
   */
  String *stringBuffer;

  /**
   * Internal buffer.
   */
  char *buffer;

  /**
   * Buffer size.
   */
  size_t bufferLength;

  /**
   * Available bytes in buffer.
   */
  size_t bufferAvailable;

  /**
   * Current position in buffer, modified by getChar().
   */
  size_t bufferPos;

  /**
   * EOF flag, set when bufferAvailable reaches 0 and 
   * there is no more data on the socket. 
   */
  bool eof;

  virtual size_t readNative(void *buffer, size_t count) = 0;
  virtual size_t writeNative(const void *buffer, size_t count) = 0;

};

#endif /* __ABSTRACTSOCKET_H__ */
