/* 
 * Copyright (c) 2000-2001, catchy.net. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      - Redistributions of source code must retain the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer.
 * 
 *      - Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 * 
 *      - All advertising materials mentioning features or use of this
 *      software must display the following acknowledgement: This
 *      product includes software developed by catchy.net and its
 *      contributors.
 * 
 *      - Neither name of catchy.net nor the names of its contributors
 *      may be used to endorse or promote products derived from this
 *      software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY CATCHY.NET AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL CATCHY.NET OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * 
 * $Id: ContentSinkResult.cpp,v 1.6 2001/03/09 10:21:17 pascal Exp $ 
 */

#include "OutOfMemoryException.h"
#include "ContentSinkResult.h"
#include "String.h"

#include <stdlib.h>
#include <string.h>

/**
 * Creates a new ContentSinkResult.
 */
ContentSinkResult::ContentSinkResult() {

  mimeType = new String("");
  content = NULL;
  contentLength = 0;

}

/**
 * Destroys the ContentSinkResult, freeing all memory held by 
 * buffers and other data in the ContentSinkResult.
 */
ContentSinkResult::~ContentSinkResult() {

  delete mimeType;
  if(content)
    free(content);

}

/**
 * Don't use; will change.
 */
void ContentSinkResult::setMimeType(const String *mt) {

  mimeType->clear();
  mimeType->append((String *) mt);

}

/**
 * Don't use; will change.
 */
void ContentSinkResult::setMimeType(const char *c) {
  
  delete mimeType;
  mimeType = new String(c);

}

/**
 * Stores a copy of the given string.
 * @throws OutOfMemoryException if there was not enough memory to
 * create a copy of s.
 * @param s the string to store.
 */
void ContentSinkResult::setContent(const char *s) {
  char *s2 = strdup(s);

  if(s2) {

    setContent(s2, strlen(s2));

  } else {

    throw OutOfMemoryException();

  }

}

/**
 * Stores a buffer pointer previously allocated with malloc() and 
 * the corresponding length. Any previously set buffer is freed.
 * @param buffer the buffer pointer to store.
 * @param length the number of valid bytes of data in the buffer.
 */
void ContentSinkResult::setContent(void *buffer, size_t length) {

  if(content)
    free(content);

  content = buffer;
  contentLength = length;

}

/**
 * Returns a pointer to a buffer previously set with setContent().
 * On exit, the supplied pointers are filled with. *buffer may be 
 * NULL if setContent() was not called yet. The caller may modify
 * the buffer but should not release the memory it occupies.
 * @param buffer pointer to a buffer pointer.
 * @param length pointer to a size_t value giving the size .
 */
void ContentSinkResult::getContent(void **buffer, size_t *length) const {
  
  *buffer = content;
  *length = contentLength;

}

/**
 * Don't use; will change.
 */
const String *ContentSinkResult::getMimeType() const {

  return mimeType;

}


