/* 
 * Copyright (c) 2000-2001, catchy.net. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *      - Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 * 
 *      - Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 * 
 *      - All advertising materials mentioning features or use of this software
 *      must display the following acknowledgement: This product includes
 *      software developed by catchy.net and its contributors.
 * 
 *      - Neither name of catchy.net nor the names of its contributors may
 *      be used to endorse or promote products derived from this software
 *      without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY CATCHY.NET AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL CATCHY.NET OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * $Id: Garbage.cpp,v 1.3 2001/01/27 14:36:40 wouter Exp $
 */

#include <stdlib.h>
#include <string.h>

#include "Garbage.h"

/**
 * Duplicates a string (like strdup()), but also marks
 * it as garbage for future garbage collection.
 * @param result pointer to a garbage table.
 * @param s the string to duplicate.
 * @return a pointer to the duplicated string.
 */
char *GarbageStrDup(GarbageTable *garbageTable, char *s) {
  char *s2 = strdup(s);
  GarbageAdd(garbageTable, s2);
  return s2;
}


/**
 * Remembers a pointer to a piece of memory allocated with
 * malloc() for future garbage collection.
 * @param result pointer to a GarbageTable.
 * @param garbage a pointer to the garbage.
 * @return 0 if there is insufficient memory, non-zero otherwise.
 */
int GarbageAdd(GarbageTable *garbageTable, void *garbage) {
  garbageTable->count++;
  garbageTable->table = (void **) realloc(garbageTable->table,
					  sizeof(void *) * 
					  garbageTable->count);
  if(!garbageTable->table)
    return 0;

  garbageTable->table[garbageTable->count - 1] = garbage;
  
  return 1;
}

/**
 * Releases a garbage table (and all the pointers it stores).
 * @param garbageTable the garbage table to delete.
 * @return 0 on failure, non-zero on success.
 */
int GarbageRelease(GarbageTable *garbageTable) {
  int i;

  if(garbageTable) {

    for(i = 0; i < garbageTable->count; i++) 
      free(garbageTable->table[i]);
    free(garbageTable->table);

  }

  return 1;
}
