#ifndef __SOCKET_H__
#define __SOCKET_H__

#include <sys/types.h> /* size_t */

#include "AbstractSocket.h"

class Socket : public AbstractSocket {

public:
  /**
   * Opens a connection to the specified port on the specified host.
   * @throws Exception if an error occurs.  
   */
  Socket(const char *hostname, unsigned short port);

  /**
   * Destroys the socket.
   */
  ~Socket();
  
private:
  /**
   * Unix socket descriptor.
   */
  int socketDescriptor;

  /**
   * Must read count bytes into buffer and return the number of bytes
   * actually read, or 0 if EOF is reached, or -1 on error. Commented
   * here because the implementation is platform dependant. For
   * information on the implementation, see the relevant Socket.cpp file.  
   */
  size_t readNative(void *buffer, size_t count);

  /**
   * Must write count bytes from buffer and return the number of bytes
   * actually written, or 0 if EOF is reached, or -1 on error. Commented
   * here because the implementation is platform dependant. For
   * information on the implementation, see the relevant Socket.cpp file.  
   */
  size_t writeNative(const void *buffer, size_t count);

};

#endif /* __SOCKET_H__ */
