/* 
 * Copyright (c) 2000-2001, catchy.net. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      - Redistributions of source code must retain the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer.
 * 
 *      - Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 * 
 *      - All advertising materials mentioning features or use of this
 *      software must display the following acknowledgement: This
 *      product includes software developed by catchy.net and its
 *      contributors.
 * 
 *      - Neither name of catchy.net nor the names of its contributors
 *      may be used to endorse or promote products derived from this
 *      software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY CATCHY.NET AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL CATCHY.NET OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * 
 * $Id: StringHash.cpp,v 1.5 2001/03/09 14:49:01 pascal Exp $ 
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "String.h"
#include "StringHash.h"

#ifdef WIN32
#define strcasecmp(a, b) stricmp(a, b)
#endif

StringHash::StringHash() {

  numberOfValues = 0;
  data = NULL;

}

StringHash::~StringHash() {
  int i;

  for(i = 0; i < numberOfValues; i += 2) {

    delete data[i];
    delete data[i + 1];

  }
  
}

int StringHash::getSize() const {
  
  return numberOfValues / 2;

}

void StringHash::put(const String *key, const String *value) {
  String **np;

  np = (String **) realloc(data, (numberOfValues + 2) * sizeof(String *));

  if(np) {

    data = np;
    data[numberOfValues] = new String(key->getPtr());
    data[numberOfValues + 1] = new String(value->getPtr());
    numberOfValues += 2;

  }

}

const String *StringHash::get(const String *key) const {
  int i;

  for(i = 0; i < numberOfValues; i += 2) {

    if(data[i]->equals(key->getPtr()))
      return data[i + 1];

  }

  return NULL;

}

const String *StringHash::getCaseInsensitive(const String *key) const {
  int i;

  for(i = 0; i < numberOfValues; i += 2) {

    if(strcasecmp(data[i]->getPtr(), key->getPtr()) == 0)
      return data[i + 1];

  }

  return NULL;

}

void StringHash::remove(const String *key) {

  printf("%s:%d: remove not implemented.\n", __FILE__, __LINE__);

}

