/* 
 * Copyright (c) 2000-2001, catchy.net. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *      - Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 * 
 *      - Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 * 
 *      - All advertising materials mentioning features or use of this software
 *      must display the following acknowledgement: This product includes
 *      software developed by catchy.net and its contributors.
 * 
 *      - Neither name of catchy.net nor the names of its contributors may
 *      be used to endorse or promote products derived from this software
 *      without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY CATCHY.NET AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL CATCHY.NET OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * $Id: GUIPeer.h,v 1.7 2001/02/26 06:38:20 pascal Exp $
 */

#ifndef __GUIPEER_H__
#define __GUIPEER_H__

/**
 * This class is a base class for platform dependent classes that
 * represent elements in the wml render window that can interact with
 * the user,such as input fields or select boxes. The platform 
 * dependent class can make use of a pointer that is declared in this 
 * class. When the element in the wml render window is no longer 
 * displayed, because another card or deck is loaded, this pointer is
 * deleted. In the destructor of the platform dependent class, for 
 * instance WindowsInputElementPeer::~WindowsInputElementPeer(), the 
 * platform dependent code can clean up the memory used for the 
 * element, for instance:
 * \code DestroyWindow(hWnd); \endcode
 */
class GUIPeer {

public:

  virtual ~GUIPeer() {};

/**
 * Returns the height of the element in the render window
 * @return the height of the element in the render window
 */
  int getHeight() {return height;}

/**
 * Returns the width of the element in the render window
 * @return the width of the element in the render window
 */
  int getWidth() {return width;}

/**
 * Returns the platform dependent pointer
 * @return the platform dependent pointer
 */
  void *getPlatformDependentPointer() {return pointer;}

protected:

  /* member data */
  int width;      /**< the width of the element */
  int height;     /**< the height of the element */
  void *pointer;  /**< the pointer to a platform dependent object */

};

#endif
