/* 
 * Copyright (c) 2000-2001, catchy.net. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *      - Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 * 
 *      - Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 * 
 *      - All advertising materials mentioning features or use of this software
 *      must display the following acknowledgement: This product includes
 *      software developed by catchy.net and its contributors.
 * 
 *      - Neither name of catchy.net nor the names of its contributors may
 *      be used to endorse or promote products derived from this software
 *      without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY CATCHY.NET AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL CATCHY.NET OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * $Id: UpdateWindowList.cpp,v 1.10 2001/01/27 14:36:40 wouter Exp $
 */

#include <malloc.h>

#include "UpdateWindowList.h"
#include "../lib/String.h"

/** 
 * Constructs a new UpdateWindowList. This is a FIFO list containing
 * actions with arguments.
 */
UpdateWindowList::UpdateWindowList() {
}

/**
 * Delete the object.
 */
UpdateWindowList::~UpdateWindowList() {

  while(argumentList.getSize())
    delete (String *)argumentList.removeLastAdded();

}

/** 
 * This function tests the code of this class
 *
 * @return      true when all tests succeed, otherwise false
 */
bool UpdateWindowList::test() {

  UpdateWindowList uwp;
  String s("hello");
  
  if(uwp.getFirstAction() != -1) 
    return false;

  uwp.deleteFirst();
  
  uwp.add((action)3,&s);
  if(uwp.getFirstAction() != (action)3)
    return false;
  if(!uwp.getFirstArgument()->equals("hello"))
    return false;

  s.clear();
  s.append("hello2");
  uwp.add((action)4,&s);

  if(uwp.getFirstAction() != (action)3)
    return false;
  if(!uwp.getFirstArgument()->equals("hello"))
    return false;

  uwp.deleteFirst();

  if(uwp.getFirstAction() != (action)4)
    return false;
  if(!uwp.getFirstArgument()->equals("hello2"))
    return false;

  return true;

}

/** 
 * Adds an action to the update window list.
 * @param action   the UpdateWindowList::action that must be performed
 * @param argument the argument which belongs to the action
 */
void UpdateWindowList::add(action a, String *argument) {

  actionList.add((int)a);
  argumentList.add(new String(argument));

}

/** 
 * Gets the action of the first element in the list. Returns -1 when
 * the list is empty.
 *
 * @return the UpdateWindowList::action of the first element in the
 * list, returns -1 when the list is empty.
 */
UpdateWindowList::action UpdateWindowList::getFirstAction() {

  if(actionList.getSize() == 0)
    return actionNoMoreActions;

  return (action) actionList.getFirstAdded();

}

/** 
 * Returns the argument of the first element in the list. Returns NULL
 * when the list is empty.
 *
 * @returns the argument of the first element in the list, returns NULL
 * when the list is empty.
 */
String *UpdateWindowList::getFirstArgument() {

  return (String *)argumentList.getFirstAdded();

}

/** 
 * Deletes the first element.
 */
void UpdateWindowList::deleteFirst() {

  actionList.removeFirstAdded();
  delete (String *)argumentList.removeFirstAdded();
  
}
