/*
 * Copyright (c) 2000-2001, catchy.net. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *      - Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *      - Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *
 *      - All advertising materials mentioning features or use of this software
 *      must display the following acknowledgement: This product includes
 *      software developed by catchy.net and its contributors.
 *
 *      - Neither name of catchy.net nor the names of its contributors may
 *      be used to endorse or promote products derived from this software
 *      without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY CATCHY.NET AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL CATCHY.NET OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: UpdateWindowList.h,v 1.17 2001/03/22 04:56:48 pascal Exp $
 */

#ifndef __UPDATEWINDOWLIST_H__
#define __UPDATEWINDOWLIST_H__

class String;

#include "IntList.h"
#include "../lib/PointerList.h"

/**
 * A class representing a list of things that must happen to the window.
 * For example:
 * \code
 *  UpdateWindowList *uwl = new UpdateWindowList();
 *  uwl->add(3,"123");
 * \endcode
 */
class UpdateWindowList {

public:
  enum action {

      /** means that there are no actions left in the list */
      actionNoMoreActions = -1,

      /** means that the uri of the WML window changed, new uri is in
          the argument */
      actionURIChanged,

      /** sets or clears the horizontal scroll bar,the argument can be
          "on" or "off" */
      actionHorizontalScrollBar,

      /** sets the position of the horizontal scroll bar */
      actionHorizontalScrollBarPos,

      /** sets the maximum position of the horizontal scroll bar*/
      actionHorizontalScrollBarRange,

      /** sets the size of the horizontal scroll bar*/
      actionHorizontalScrollBarPage,

      /** sets or clears the vertical scroll bar,the argument can be
          "on" or "off" */
      actionVerticalScrollBar,

      /** sets the position of the vertical scroll bar */
      actionVerticalScrollBarPos,

      /** sets the maximum position of the vertical scroll bar*/
      actionVerticalScrollBarRange,

      /** sets the size of the vertical scroll bar*/
      actionVerticalScrollBarPage,

      /** change the back button label with the label in the argument */
      actionChangeBackButtonLabel,

      /** add a button with the label in the argument */
      actionAddButtonLabel,

      /** delete all buttons */
      actionClearButtons,

      /** change the text of the output window with the text int the
          argument */
      actionSetOutputWindowText,

      /** clear the text of the output window */
      actionClearOutputWindowText,

      /** changes the mouse pointer to the hand pointer,
          typically when the mouse is positioned on a link */
      actionSetHandMousePointer,

      /** changes the mouse pointer to the normal (arrow) pointer,
          typically when the mouse is NOT positioned on a link */
      actionSetNormalMousePointer,

      /** Enables / disables the 'Back' button. The argument can be
          either "on" or "off". */
      actionBackButton,

      /** Enables / disables the 'Forward' button. The argument can be
          either "on" or "off". */
      actionForwardButton

  };

  UpdateWindowList();
  ~UpdateWindowList();
  static bool test();
  void add(action a, String *argument);
  action getFirstAction();
  String *getFirstArgument();
  void deleteFirst();

private:

  /* member data */
  IntList actionList;       /**< the list of actions */
  PointerList argumentList; /**< the list of arguments */

};

#endif
