/* 
 * Copyright (c) 2000-2001, catchy.net. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      - Redistributions of source code must retain the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer.
 * 
 *      - Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 * 
 *      - All advertising materials mentioning features or use of this
 *      software must display the following acknowledgement: This
 *      product includes software developed by catchy.net and its
 *      contributors.
 * 
 *      - Neither name of catchy.net nor the names of its contributors
 *      may be used to endorse or promote products derived from this
 *      software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY CATCHY.NET AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL CATCHY.NET OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * 
 * $Id: WBMP.cpp,v 1.13 2001/03/23 17:06:48 wouter Exp $ */

#include "WBMP.h"
#include "../lib/String.h"
#include "../lib/URI.h"
#include "../lib/ContentSink.h"
#include "WMLWindow.h"

/** 
 * Constructs a new WBMP object.
 * @param ww the WMLWindow of the picture
 * @param s the src argument of the img tag
 */
WBMP::WBMP(WMLWindow *ww, String *src) {
  char *f;
  size_t length;

  width = 0;
  height = -1;
  imageArray = NULL;

  URI imageURI(src->getPtr());
  URI *resolvedURI = URI::resolve(ww->getURI(),&imageURI);

  ContentSinkResult *csr = ww->getContentSink()->get(resolvedURI, NULL);
  delete resolvedURI;


  csr->getContent((void **) &f, &length);

  if(f == NULL) {

    delete csr;
    return;

  }

  if(f[0] != 0 || f[1] != 0) {

    delete csr;
    return;

  }

  size_t offset = 2; /* the number of bytes that have been read */
  
  /* set Width: */

  if(f[offset] & 0x80) { /* detect multi-byte */

    if(f[offset+1] & 0x80) {
      /* multi byte of more than two bytes, which means a picture
         with width > 16000 those pictures are ignored for now */
      delete csr;
      return;

    }

    width = ((int)f[offset] & 0x7f) * 128  + f[offset+1];
    offset++;

  }
  else
    width = (int)f[offset];

  offset++;

  /* set Height: */
  
  if(f[offset] & 0x80) { /* detect multi-byte */

    if(f[offset+1] & 0x80) {

      /* multi byte of more than two bytes, which means a picture
         with height > 16000 
 those pictures are ignored for now */
      
      width = 0;
      delete csr;
      return;
      
    }

    height = ((int)f[offset] & 0x7f) * 128  + f[offset+1];
    offset++;

  }
  else
    height = (int)f[offset];

  offset++;

  /* check if the length is long enough for Width and Height: */

  int bytes;

  if(width%8) 
    bytes = (width/8 + 1) * height;
  else 
    bytes = width/8 * height;

  if(offset + bytes > length) {
    
    delete csr;
    return;

  }

  /* Start reading image data in the array */

  imageArray = new char [bytes];

  for(bytes--; bytes >= 0; bytes--) {

    imageArray[bytes] = f[offset + bytes];

  }

  delete csr;

  
}

/**
 * Delete the object.
 */
WBMP::~WBMP() {

  delete imageArray;

}

/**
 * This function tests the code of this class
 *
 * @return  true when all tests succeed, otherwise false
 */
bool WBMP::test() {

  /*  String fileName("../source/tests/small.wbmp");
  
      WBMP smallpicture(&fileName);
      if(smallpicture.getWidth()  != 14) return false;
      if(smallpicture.getHeight() != 11) return false;

      fileName.clear();
      fileName.append("../source/tests/horizontalMultiByte.wbmp");

      WBMP horizontalMultiBytePicture(&fileName);
      if(horizontalMultiBytePicture.getWidth()  != 350) return false;
      if(horizontalMultiBytePicture.getHeight() != 50)  return false;

      fileName.clear();
      fileName.append("../source/tests/verticalMultiByte.wbmp");

      WBMP verticalMultiBytePicture(&fileName);
      if(verticalMultiBytePicture.getWidth()  != 50)  return false;
      if(verticalMultiBytePicture.getHeight() != 350) return false;

      fileName.clear();
      fileName.append("../source/tests/bothMultiByte.wbmp");

      WBMP bothMultiBytePicture(&fileName);
      if(bothMultiBytePicture.getWidth()  != 350) return false;
      if(bothMultiBytePicture.getHeight() != 430) return false;
  */
  return true;

}

/**
 * Returns the width of the image.
 * @return the width of the image
 */
int WBMP::getWidth() {

  return width;

}

/**
 * Returns the height of the image.
 * @return the height of the image
 */
int WBMP::getHeight() {

  return height;

}

/**
 * Returns the image data. This is a monochrome bitmap format.
 * @return the image data
 */
char *WBMP::getImageData() {

  return imageArray;

}
