/* 
 * Copyright (c) 2000-2001, catchy.net. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *      - Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 * 
 *      - Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 * 
 *      - All advertising materials mentioning features or use of this software
 *      must display the following acknowledgement: This product includes
 *      software developed by catchy.net and its contributors.
 * 
 *      - Neither name of catchy.net nor the names of its contributors may
 *      be used to endorse or promote products derived from this software
 *      without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY CATCHY.NET AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL CATCHY.NET OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * $Id: WMLGUIElement.cpp,v 1.6 2001/03/08 16:18:50 wouter Exp $
 */

#include "WMLGUIElement.h"

/**
 * Construct a new WMLGUIElement. 
 * @param gp a platform dependent object that is subclassed from G
 * UIPeer, this object is deleted when the element is no longer needed.
 * @param id the identifier of this element, every element in the wml
 *        renderer that accepts user input has an unique identifier
 */
WMLGUIElement::WMLGUIElement(GUIPeer* gp, int identifier) {

  peer = gp;
  id = identifier;

}

/**
 * Deletes the object. In this function the platform pointer 
 * is deleted.
 */
WMLGUIElement::~WMLGUIElement() {

  delete peer; 

}

/**
 * Returns the identifier of this element. Every element in the wml
 * renderer that accepts user input has an unique identifier
 * @return the identifier of this input element
 */
int WMLGUIElement::getId() {

  return id;

}

/**
 * Returns the x coordinate of the element.
 * @return the x coordinate of the element
 */
int WMLGUIElement::getXCoordinate() {

  return x;

}

/**
 * Returns the y coordinate of the element.
 * @return the y coordinate of the element
 */
int WMLGUIElement::getYCoordinate() {

  return y;

}

/**
 * Returns width of the element.
 * @return width of the element.
 */
int WMLGUIElement::getWidth() {

  if(peer)
    return peer->getWidth();
  return 0;

}

/**
 * Returns height of the element.
 * @return height of the element.
 */
int WMLGUIElement::getHeight() {

  if(peer) 
    return peer->getHeight();
  return 0;

}

/**
 * Sets the x coordinate of the element.
 * @param xc the new x coordinate
 */
void WMLGUIElement::setXCoordinate(int xc) {

  x = xc;

}

/**
 * Sets the y coordinate of the element.
 * @param yc the new y coordinate
 */
void WMLGUIElement::setYCoordinate(int yc) {

  y = yc;

}

/**
 * Returns the platform dependent pointer.
 * @return the platform dependent pointer
 */
GUIPeer* WMLGUIElement::getPeer() {

  return peer;

}
