/* 
 * Copyright (c) 2000-2001, catchy.net. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      - Redistributions of source code must retain the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer.
 * 
 *      - Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 * 
 *      - All advertising materials mentioning features or use of this
 *      software must display the following acknowledgement: This
 *      product includes software developed by catchy.net and its
 *      contributors.
 * 
 *      - Neither name of catchy.net nor the names of its contributors
 *      may be used to endorse or promote products derived from this
 *      software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY CATCHY.NET AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL CATCHY.NET OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * 
 * $Id: WMLWindow.h,v 1.40 2001/03/22 04:56:48 pascal Exp $ 
 */

#ifndef __WMLWINDOW_H__
#define __WMLWINDOW_H__

class WindowDrawingInterface;
class WMLTable;
class XMLCharData;
class WMLTable;
class XMLBody;
class XMLCharData;
class XMLElement;

#include "../lib/String.h"
#include "UpdateWindowList.h"
#include "../lib/ContentSink.h"
#include "../lib/Point.h"
#include "../lib/URI.h"
#include "StyleList.h"
#include "IntList.h"
#include "WMLAnchorRectangleList.h"
#include "WMLGUIElementList.h"

/** 
 * This class represents the window in wich wml content is displayed.
 * For example:
 * \code
 *   PalmInterface *pi = new PalmInterface();
 *   WMLWindow *ww = new WMLWindow((WindowDrawingInterface *) pi);
 *   ww->drawContentWindow();
 * \endcode
 */
class WMLWindow {

 public:
  
  enum style {
    /** render normally */
    styleNoStyle = 0,

    /** render with a bold font */
    styleBold = 1,

    /** render with an italic  font*/
    styleItalic = 2,

    /** render with an underlined  font*/
    styleUnderlined = 4,

    /** render with a bigger font */
    styleBig = 8,

    /** render with a smaller font */
    styleSmall = 16,

    /** render emphasized (big) */
    styleEm = styleBig,

    /** render strong (big and underlined) */
    styleStrong =  styleBig | styleUnderlined,   

    /** render a link */
    styleLink = 32,

    /** render the topmost link */
    styleToplink = 64
  };
  
  enum alignMethod {
    /** align at the left side */
    alignLeft,

    /** align at the middle */
    alignCenter,

    /** align at the right side */
    alignRight
  }; 
  
  enum button {
    /** the ok button */
    buttonOK
  };

  WMLWindow(WindowDrawingInterface *wdi);
  ~WMLWindow();
  static bool test();
  void setSource(URI *u);
  void setCard(String *s);
  void mouseMove(int x, int y);
  bool mouseClick(int x, int y);
  void pushedButton(button b);
  UpdateWindowList *getUpdates();
  void drawContentWindow();
  void scrollVertical(int i);
  void setVertical(int i);
  void scrollHorizontal(int i);
  void setHorizontal(int i);
  void setWindowDimensions(int width, int height);
  WindowDrawingInterface *getWindowDrawingInterface();
  bool getMousePointerShape();
  URI *getURI();
  ContentSink *getContentSink();
  WMLGUIElement *getImage(String *src);
  bool historyGo(int direction);

 private:
  
  void setOutputWindowText(String* s);
  void clearCardDependentMemory();
  void drawContentWindow(bool draw);
    
  void setButtonLabels();
  void setButtonLabels(XMLBody *xb);
  void addButtonLabel(XMLBody *xb);

  void displayParagraph(XMLElement *xe);
  void displayParagraph(XMLBody *xb);
  
  void displayInputElement (XMLBody *xb);
  void displaySelectElement(XMLBody *xb);
  void displayTableElement (XMLBody *xb); 
  void displayImageElement (XMLBody *xb);
  void displayParagraphText(XMLCharData *xcd);
  
  void updateCurrentLineHeight(int height);
  void updateSourceWidth(int x);
  void newLineForward();
  void newLineBackward();
  
  int additionalWordLength(XMLCharData *xcd, PointerList *xbl, StyleList *sl);
  int charDataLength(XMLCharData *xcd, int style, bool *lastCharacterWasSpace);
  int xmlLength(XMLBody* xb, StyleList* sl, bool *lastCharacterWasSpace);

  static String *getLinkTarget(XMLElement *xe);
  void getOptions(PointerList *wol, XMLBody *xb);

  void setSource(URI *u, bool updateHistory);
  void historyAdd(URI *u);
  
  /* member data */
  /** a string that contains the same string as the current
      text in the output window */
  String outputWindowText;

  /** true iff the mouse pointer must be a hand in stead of
      a arrow */
  bool handMousePointer;

  /** the parse tree of the source */
  XMLElement *parseTree;

  /** a list of things the window drawing
      interface has to do */
  UpdateWindowList updater;

  /** the current card, this is a pointer into the parsetree */
  XMLElement *currentCard;

  /** the platform dependent WindowDrawingInterface */
  WindowDrawingInterface *windowInterface;

  /** the height of the rendered source */
  int sourceHeight;

  /** the width  of the rendered source */
  int sourceWidth;

  /** the height of the window */
  int windowHeight;

  /** the width  of the window */
  int windowWidth;

  /** true iff the vertical scroll bar is used */
  bool vScrollBar;

  /** true iff the horizontal scroll bar is used */
  bool hScrollBar;

  /** the amount of pixels the content must be scrolled down */
  int vScroll;

  /** the amount of pixels the content must be scrolled to
      the right */
  int hScroll;

  /** the current line height, used to determine the amount 
      of pixels that must be skipped when a < br > occurs*/
  int currentLineHeight;

  /** a list of anchors that are located in the window */
  WMLAnchorRectangleList anchorList;

  /** the link that was last read in the wml code, used 
      to build the WMLAnchorRectangleList */
  String *lastReadLinkTarget;

  /** the top link target, used to go to this source 
      when the ok button is pressed */
  String *topLinkTarget;

  /** the amount of read links that are in the display area
      used to render the first link differently */
  int readDisplayedLinks;
  /** the list of input elements in the source */
  WMLGUIElementList inputList;

  /** the list of select elements in the source */
  WMLGUIElementList selectList;

  /** the list of select elements in the source */
  WMLGUIElementList imageList;

  /** the number of elements processed that need a GUIPeer */
  int readPeerElements;

  /** the number of images processed */
  int readImages;

  /** the position the renderer is currently at */
  Point currentLocation;

  /** true -> the renderer is only determining the size of 
      the source, so it is not displaying anything 
      false -> the renderer is displaying */
  bool determineSize;

  /** a list of xml body's that the renderer still has to
      display */
  PointerList bodyList;

  /** the list of added styles, styleList.getStyle() is
      the current style */
  StyleList styleList;

  /** the current method for aligning, possible values:
      alignLeft, alignCenter, alignRight */
  alignMethod currentAlignmentMethod;

  /** the wrapping method of the currently rendered 
      paragraph, true when the mode attribute of the p
      element equals 'wrap' otherwise false */
  bool wrap;

  /** true -> tells the next performed member that the
      next bit of text could be displayed on the next line,
      this is typically NOT true when rendering something
      like 'lond< b >on< /b >' because 'on' must be displayed
      on the same line */
  bool breakPossible;

  /** a list in which the distance from the end of the
      line to the end of the window is kept for every line */
  IntList offsetList;

  /** the content sink */
  ContentSink contentSink;

  /** the uri of this document */
  URI *uri;
  
  enum {
    /** this constant is assigned to currentLineHeight so
        that when before the paragraph end the wml programmer
        programmed a < br > this newline is not 'displayed' */
    endOfParagraph = -1,
    
  };

  /** The history. */
  PointerList *history;

  /** The current position in the history. */
  int historyIndex;

};

#endif
