/* 
 * Copyright (c) 2000-2001, catchy.net. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      - Redistributions of source code must retain the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer.
 * 
 *      - Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 * 
 *      - All advertising materials mentioning features or use of this
 *      software must display the following acknowledgement: This
 *      product includes software developed by catchy.net and its
 *      contributors.
 * 
 *      - Neither name of catchy.net nor the names of its contributors
 *      may be used to endorse or promote products derived from this
 *      software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY CATCHY.NET AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL CATCHY.NET OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * 
 * $Id: Tester.cpp,v 1.20 2001/03/23 04:12:16 pascal Exp $ 
 */

#include <stdio.h>

#include "Tester.h"
#include "../lib/MobilizerPreferences.h"
#include "../lib/URI.h"
#include "../lib/ContentSink.h"
#include "../lib/PointerList.h"
#include "../wmlparser/CharStream.h"
#include "../wmlparser/XMLAttributeList.h"
#include "../wmlparser/XMLBody.h"
#include "../wmlparser/XMLCharData.h"
#include "../wmlparser/XMLElement.h"
#include "../wmlparser/XMLTokenizer.h"
#include "../wmlparser/XMLParser.h"
#include "../wmlparser/WMLParser.h"
#include "../renderer/IntList.h"
#include "../renderer/StyleList.h"
#include "../renderer/UpdateWindowList.h"
#include "../renderer/WMLAnchorRectangleList.h"
#include "../renderer/WBMP.h"
#include "../renderer/WMLWindow.h"
#include "../wmlscript/WMLScript.h"

/** 
 * Constructs a new Tester object.
 */
Tester::Tester() {

  feedbackCounter = 0;

}

/**
 * Delete the object
 */
Tester::~Tester() {
}

/** 
 * This function does the testing. This function returns when there is
 * feedback. It returns false when all testing is done.
 *
 * @param      feedback    feedback from the testing
 * @return     false when ready, otherwise true
 */
bool Tester::doTesting(String* feedback) {
  WMLScriptTester tester;

  switch (feedbackCounter) {

  case 0:
    startTime = clock();
    
    feedback->append("testing class 'String'... ");
    break;

  case 1:
    appendAppropriateFeedback(feedback,String::test());
    break;
  
  case 2:
    feedback->append("testing class 'CharStream'... ");
    break;

  case 3: 
    appendAppropriateFeedback(feedback,CharStream::test());
    break;

  case 4:
    feedback->append("testing class 'XMLTokenizer'... ");
    break;

  case 5: 
    appendAppropriateFeedback(feedback,XMLTokenizer::test());
    break;

  case 6:
    feedback->append("testing class 'XMLAttributeList'... ");
    break;

  case 7: 
    appendAppropriateFeedback(feedback,XMLAttributeList::test());
    break;

  case 8:
    feedback->append("testing class 'XMLBody'... ");
    break;

  case 9:
    appendAppropriateFeedback(feedback,XMLBody::test());
    break;

  case 10:
    feedback->append("testing class 'XMLElement'... ");
    break;

  case 11:
    appendAppropriateFeedback(feedback,XMLElement::test());
    break;

  case 12:
    feedback->append("testing class 'XMLParser'... ");
    break;

  case 13: 
    appendAppropriateFeedback(feedback,XMLParser::test());
    break;

  case 14:
    feedback->append("testing class 'WMLParser'... ");
    break;

  case 15:
    appendAppropriateFeedback(feedback,WMLParser::test());
    break;

  case 16:
    feedback->append("testing class 'UpdateWindowList'... ");
    break;

  case 17: 
    appendAppropriateFeedback(feedback,UpdateWindowList::test());
    break;

  case 18:
    feedback->append("testing class 'XMLCharData'... ");
    break;

  case 19: 
    appendAppropriateFeedback(feedback,XMLCharData::test());
    break;

  case 20:
    feedback->append("testing class 'StyleList'... ");
    break;

  case 21: 
    appendAppropriateFeedback(feedback,StyleList::test());
    break;

  case 22:
    feedback->append("testing class 'IntList'... ");
    break;

  case 23: 
    appendAppropriateFeedback(feedback,IntList::test());
    break;

  case 24:
    feedback->append("testing class 'WMLScriptTester'... \n");
    break;
    
  case 25:
    appendAppropriateFeedback(feedback, tester.test() == 0);
    break;
    
  case 26:
    feedback->append("testing class 'WMLWindow'... ");
    break;

  case 27: 
    appendAppropriateFeedback(feedback,WMLWindow::test());
    break;

  case 28:
    feedback->append("testing class 'WMLAnchorRectangleList'... ");
    break;

  case 29: 
    appendAppropriateFeedback(feedback,WMLAnchorRectangleList::test());
    break;

  case 30:
    feedback->append("testing class 'MobilizerPreferences'... ");
    break;

  case 31: 
    appendAppropriateFeedback(feedback,MobilizerPreferences::test());
    break;

  case 32:
    feedback->append("testing class 'URI'... ");
    break;

  case 33:
    appendAppropriateFeedback(feedback, URI::test() == 0);
    break;

  case 34:
    feedback->append("testing class 'PointerList'... ");
    break;

  case 35:
    appendAppropriateFeedback(feedback, PointerList::test());
    break;

  case 36:
    feedback->append("testing class 'WBMP'... ");
    break;

  case 37:
    appendAppropriateFeedback(feedback, WBMP::test());
    break;

  case 38:
    feedback->append("testing class 'ContentSink'... ");
    break;

  case 39:
    appendAppropriateFeedback(feedback, ContentSink::test());
    break;

  default:
    {
      double duration = (double)(clock() - startTime) / CLOCKS_PER_SEC;
      feedback->append("in ");
      feedback->append(duration);
      feedback->append(" seconds.\r\n");

    }
    return false;
 
  }
  feedbackCounter++;
  return true;

}

/** 
 * Appends appropriate feedback, used by doTesting
 * @param feedback the string to append to
 * @param test     the output of the test, true means succesfull, false means failed
 */
void Tester::appendAppropriateFeedback(String* feedback, bool test) {
  
  if(test) {

    feedback->append("passed.\r\n");
  
  } else {

    feedback->append("failed.\r\n");
  
  }
  return;

}
