/* 
 * Copyright (c) 2000-2001, catchy.net. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *      - Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 * 
 *      - Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 * 
 *      - All advertising materials mentioning features or use of this software
 *      must display the following acknowledgement: This product includes
 *      software developed by catchy.net and its contributors.
 * 
 *      - Neither name of catchy.net nor the names of its contributors may
 *      be used to endorse or promote products derived from this software
 *      without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY CATCHY.NET AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL CATCHY.NET OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * $Id: CharStream.h,v 1.11 2001/01/27 14:36:40 wouter Exp $
 * 
 * Reviewed  3-12-2000 ph
 * Reviewed 16-01-2001 wk
 */

#ifndef __CHARSTREAM_H__
#define __CHARSTREAM_H__

class String;

/** 
 * A class representing a character stream.
 * For example:
 * \code
 *  CharStream* cs = new CharStream(source);
 *  char c = cs->getNextChar();
 * \endcode
 */
class CharStream {

public:

  CharStream(String *source);
  ~CharStream();
  static bool test();
  char getNextChar();
  void pushBack();
  int getLineNumber();
  bool isEndOfFile();
  void savePosition();
  void recallPosition();

private:

  /* member data */
  /** pointer to the beginning of the buffer */
  char *begin;

  /** pointer to the current character in the buffer */
  char *current;

  /** pointer to the last character of the buffer */
  char *last;

  /** pointer to the saved position */
  char *recall;

  /** the current line number */
  int currentLineNumber; 

};

#endif
