/* 
 * Copyright (c) 2000-2001, catchy.net. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      - Redistributions of source code must retain the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer.
 * 
 *      - Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 * 
 *      - All advertising materials mentioning features or use of this
 *      software must display the following acknowledgement: This
 *      product includes software developed by catchy.net and its
 *      contributors.
 * 
 *      - Neither name of catchy.net nor the names of its contributors
 *      may be used to endorse or promote products derived from this
 *      software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY CATCHY.NET AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL CATCHY.NET OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * 
 * $Id: XMLParser.h,v 1.8 2001/01/27 14:36:41 wouter Exp $ 
 * 
 * Reviewed 4-12-2000 ph
 */

#ifndef __XMLPARSER_H__
#define __XMLPARSER_H__

class String;
class XMLElement;

#include "XMLTokenizer.h"

/** 
 * This class represents a XML Parser. <br>
 * Typical use:
 * \code
 *   XMLElement *xe = new XMLElement();
 *   XMLParser *xp = new XMLParser(source, xe);
 * \endcode
 *
 */
class XMLParser {

 public:

  XMLParser(String *source, XMLElement *topElement);
  ~XMLParser();
  static bool test();
  String *getTopElementName();
  String *getErrorMessage();
  void appendToErrorString(char *e);
  void appendToErrorString(int i);
  void generateError(char *e);
  void generateError();
  void generateErrorMentionToken(char *e);
  void generateUnexpectedTokenError();
  void readCommentsAndProcessingInstructions();

 private:

  void readXMLDeclaration();
  void readDocTypeDeclaration();
  void makeErrorText(String *s, char *message, XMLTokenizer::xmlToken token);

  /** the error message */
  String *xmlErrorMessage;

  /** the tokenizer */
  XMLTokenizer *xmlTokenizer;

  /** the name of the top element according to document type declaration. */
  String *xmlTopElementName;   

};

#endif
