/* 
 * Copyright (c) 2000-2001, catchy.net. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      - Redistributions of source code must retain the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer.
 * 
 *      - Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 * 
 *      - All advertising materials mentioning features or use of this
 *      software must display the following acknowledgement: This
 *      product includes software developed by catchy.net and its
 *      contributors.
 * 
 *      - Neither name of catchy.net nor the names of its contributors
 *      may be used to endorse or promote products derived from this
 *      software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY CATCHY.NET AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL CATCHY.NET OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * 
 * $Id: XMLTokenizer.h,v 1.10 2001/01/27 14:36:41 wouter Exp $ 
 *
 * Reviewed 4-12-2000 ph (enum, naming)
 */

#ifndef __XMLTOKENIZER_H__
#define __XMLTOKENIZER_H__

class CharStream;
class String;

/** This class represents a XML Tokenizer.
  * Typical use:
  * \code
  *   XMLTokenizer *xt = new XMLTokenizer(source);
  *   if(xt->getNextToken() == XMLTokenizer::XMLOpen) ...
  * \endcode
  *
  */
class XMLTokenizer {

public:
  enum xmlToken { 
    tokenXMLOpen,                /**< "<?xml"                             */   
    tokenXMLClose,               /**< "?>"                                */  
    tokenDocTypeOpen,            /**< "<!DOCTYPE"                         */
    tokenCDataOpen,              /**< "<![CDATA["                         */
    tokenCloseTag,               /**< ">"                                 */ 
    tokenCloseEmptyTag,          /**< "/>"                                */
    tokenName,                   /**< like "card", "id"                   */
    tokenEquals,                 /**< "="                                 */  
    tokenStringConstant,         /**< ""a single or double quoted string""*/  
    tokenEndlessString,          /**< ".........                          */
    tokenTagOpen,                /**< "<image", "<street", ...            */
    tokenTagClose,               /**< "</image>", "</street>", ...        */
    tokenComment,                /**< "< !--this is comment-->"           */
    tokenProcessingInstruction,  /**< "<?processing instruction?>"        */  
    tokenUnknownToken,           /**< everything else                     */ 
    tokenNothing                 /**< nothing */
  }; 

  XMLTokenizer(String *source);
  ~XMLTokenizer();
  static bool test();
  xmlToken getNextToken();
  xmlToken getLastReadToken();
  String *getTokenAsString();
  bool readCharacterData(String *s);
  int getTokenStartLineNumber();
  bool isEndOfFile();
  int getLineNumber();
  void pushbackToken();
  void readCDataSection(String *s);

private:

  void readComment();
  void readProcessingInstruction();
  static bool isNameCharacter(char c);
  xmlToken readNewToken();
  xmlToken match(char* s, xmlToken);

  /* member data */
  /** the char stream this tokenizer uses */
  CharStream *characterStream;

  /** the last token, used for getTokenAsString() */
  String *tokenAsString;

  /** the line number where the tokenizer started reading the token */
  int tokenStartLineNumber;

  /** the last read token, used by getLastReadToken() */
  xmlToken lastReadToken;

};

#endif
