/* 
 * Copyright (c) 2000-2001, catchy.net. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *      - Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 * 
 *      - Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 * 
 *      - All advertising materials mentioning features or use of this software
 *      must display the following acknowledgement: This product includes
 *      software developed by catchy.net and its contributors.
 * 
 *      - Neither name of catchy.net nor the names of its contributors may
 *      be used to endorse or promote products derived from this software
 *      without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY CATCHY.NET AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL CATCHY.NET OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * $Id: WMLScriptFunctionParameters.h,v 1.5 2001/01/27 14:36:41 wouter Exp $
 */

#ifndef __WMLSCRIPTFUNCTIONPARAMETERS_H__
#define __WMLSCRIPTFUNCTIONPARAMETERS_H__

/**
 * This object is used to pass parameters to a WMLScript function.
 * 
 * Typical use:
 * \code
 * WMLScriptParser parser = WMLScriptParser();

 * try {
 *
 *   parseContext = parser.parse("extern function multiply(a, b) { return a * b; };");
 *   WMLScriptInterpreter interpreter(parseContext);
 *   WMLScriptFunctionParameters parameters;
 *   parameters.add(3);
 *   parameters.add(4);
 *   val = interpreter.call("multiply", &parameters); // val = 12
 *
 * } catch(WMLScriptException& e) {
 *
 *   ... error handling ...
 *
 * }
 * \endcode
 */
class WMLScriptFunctionParameters {
 public:
  WMLScriptFunctionParameters();
  ~WMLScriptFunctionParameters();

  void add(bool b);
  void add(char *s);
  void add(int i);
  void add(double d);
  void add(WMLScriptValueObject *v);
  void insert(bool b, int before);
  void insert(char *s, int before);
  void insert(int i, int before);
  void insert(double d, int before);
  void insert(WMLScriptValueObject *v, int before);
  int size();
  WMLScriptValueObject *valueAt(int i);

 private:
  int numberOfParameters;
  WMLScriptValueObject **parameters;
};

#endif
