/* 
 * Copyright (c) 2000-2001, catchy.net. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *      - Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 * 
 *      - Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 * 
 *      - All advertising materials mentioning features or use of this software
 *      must display the following acknowledgement: This product includes
 *      software developed by catchy.net and its contributors.
 * 
 *      - Neither name of catchy.net nor the names of its contributors may
 *      be used to endorse or promote products derived from this software
 *      without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY CATCHY.NET AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL CATCHY.NET OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * $Id: WMLScriptLibrary.cpp,v 1.7 2001/01/27 14:36:41 wouter Exp $
 */

#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>

#include "../lib/snprintf.h"

#include "WMLScriptValueObject.h"
#include "WMLScriptFunctionParameters.h"
#include "WMLScriptLibrary.h"
#include "WMLScriptWrongArityException.h"

/**
 * Constructs a WMLScriptLibrary.
 */
WMLScriptLibrary::WMLScriptLibrary() {
}

/**
 * Destructs the WMLScriptLibrary.
 */
WMLScriptLibrary::~WMLScriptLibrary() {
}

/**
 * Given a function name, returns the ID of that function in
 * the library. Function name matching must be case-sensitive.
 * @param name the function name to look up.
 * @return the function ID of the library function or -1
 * if it does not exist.
 */
int WMLScriptLibrary::getFunctionID(const char *name) const {
  return 0;
}

/**
 * Executes a function specified by the given function ID.
 * @param id the function ID to invoke.
 * @param parameters the parameters to invoke the function with.
 * @throws WMLScriptWrongArityException
 * if the number of parameters does not match the expected
 * number of parameters.
 * @throws WMLScriptUnknownLibraryFunctionException
 * if the given function ID does not refer to a known function.
 * @throws WMLScriptException
 * if the implementation returns some other error.
 * @return the return value of the library function.
 */
WMLScriptValueObject *WMLScriptLibrary::call(int id, WMLScriptFunctionParameters *parameters) {
  return NULL;
}

/**
 * This function returns the name of this library.
 * @return pointer to the name of this library.
 */
const char *WMLScriptLibrary::getName() const {
  return NULL;
}

/**
 * Checks whether the number of parameters in a parameter list
 * matches the expected number of parameters; otherwise, creates
 * an error message and throws an exception.
 * @param parameters the parameter list.
 * @param required_params the expected number of parameters.
 * @param function the function name.
 */
void WMLScriptLibrary::paramCheck(WMLScriptFunctionParameters *parameters, int required_params, char *function) {
  char s[MSG_SIZE];

  if(parameters->size() != required_params) {

    snprintf(s, MSG_SIZE, "invalid number of parameters to %s.%s: expected %d, got %d", 
	    getName(), function, required_params, parameters->size());
    throw WMLScriptWrongArityException(s);

  }

}

