/* 
 * Copyright (c) 2000-2001, catchy.net. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *      - Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 * 
 *      - Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 * 
 *      - All advertising materials mentioning features or use of this software
 *      must display the following acknowledgement: This product includes
 *      software developed by catchy.net and its contributors.
 * 
 *      - Neither name of catchy.net nor the names of its contributors may
 *      be used to endorse or promote products derived from this software
 *      without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY CATCHY.NET AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL CATCHY.NET OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * $Id: WMLScriptLibraryLang.h,v 1.6 2001/01/27 14:36:41 wouter Exp $
 */

#ifndef __WMLSCRIPTLIBRARYLANG_H__
#define __WMLSCRIPTLIBRARYLANG_H__

#include "WMLScriptLibrary.h"

#define FUNC_ABS 0
#define FUNC_MIN 1
#define FUNC_MAX 2
#define FUNC_PARSEINT 3
#define FUNC_PARSEFLOAT 4
#define FUNC_ISINT 5
#define FUNC_ISFLOAT 6
#define FUNC_MAXINT 7
#define FUNC_MININT 8
#define FUNC_FLOAT 9
#define FUNC_EXIT 10
#define FUNC_ABORT 11
#define FUNC_RANDOM 12
#define FUNC_SEED 13
#define FUNC_CHARACTERSET 14

/**
 * This class implements the Lang library as specified
 * by WAP-194-WMLScript Standard Libraries Specification [June 2000].
 *
 * Two functions in this library are very special, namely the
 * exit and abort functions. These functions, when executed,
 * result in the interpreter terminating with an exception.
 * If there is any chance that the interpreter will be executing
 * these functions, the embedding code should take care to catch the
 * resulting exceptions.
 */
class WMLScriptLibraryLang : public WMLScriptLibrary {
 public:
  WMLScriptLibraryLang();
  ~WMLScriptLibraryLang();

  int getFunctionID(const char *name) const;
  WMLScriptValueObject *call(int id, WMLScriptFunctionParameters *parameters);
  const char *getName() const;
  
 private:
  WMLScriptValueObject *funcAbs(WMLScriptFunctionParameters *parameters);
  WMLScriptValueObject *funcMin(WMLScriptFunctionParameters *parameters);
  WMLScriptValueObject *funcMax(WMLScriptFunctionParameters *parameters);
  WMLScriptValueObject *funcParseInt(WMLScriptFunctionParameters *parameters);
  WMLScriptValueObject *funcParseFloat(WMLScriptFunctionParameters *parameters);
  WMLScriptValueObject *funcIsInt(WMLScriptFunctionParameters *parameters);
  WMLScriptValueObject *funcIsFloat(WMLScriptFunctionParameters *parameters);
  WMLScriptValueObject *funcMaxInt(WMLScriptFunctionParameters *parameters);
  WMLScriptValueObject *funcMinInt(WMLScriptFunctionParameters *parameters);
  WMLScriptValueObject *funcFloat(WMLScriptFunctionParameters *parameters);
  void funcExit(WMLScriptFunctionParameters *parameters);
  void funcAbort(WMLScriptFunctionParameters *parameters);
  WMLScriptValueObject *funcRandom(WMLScriptFunctionParameters *parameters);
  WMLScriptValueObject *funcSeed(WMLScriptFunctionParameters *parameters);
  WMLScriptValueObject *funcCharacterSet(WMLScriptFunctionParameters *parameters);
};

#endif
