/* 
 * Copyright (c) 2000-2001, catchy.net. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *      - Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 * 
 *      - Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 * 
 *      - All advertising materials mentioning features or use of this software
 *      must display the following acknowledgement: This product includes
 *      software developed by catchy.net and its contributors.
 * 
 *      - Neither name of catchy.net nor the names of its contributors may
 *      be used to endorse or promote products derived from this software
 *      without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY CATCHY.NET AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL CATCHY.NET OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * $Id: WMLScriptLibraryString.h,v 1.6 2001/01/27 14:36:41 wouter Exp $
 */

#ifndef __WMLSCRIPTLIBRARYSTRING_H__
#define __WMLSCRIPTLIBRARYSTRING_H__

#include "WMLScriptLibrary.h"

#define FUNC_LENGTH 0
#define FUNC_ISEMPTY 1
#define FUNC_CHARAT 2
#define FUNC_SUBSTRING 3
#define FUNC_FIND 4
#define FUNC_REPLACE 5
#define FUNC_ELEMENTS 6
#define FUNC_ELEMENTAT 7
#define FUNC_REMOVEAT 8
#define FUNC_REPLACEAT 9
#define FUNC_INSERTAT 10
#define FUNC_SQUEEZE 11
#define FUNC_TRIM 12
#define FUNC_COMPARE 13
#define FUNC_TOSTRING 14
#define FUNC_FORMAT 15

/**
 * This class implements the String library as specified
 * by WAP-194-WMLScript Standard Libraries Specification [June 2000].
 */
class WMLScriptLibraryString : public WMLScriptLibrary {
 public:
  WMLScriptLibraryString();
  ~WMLScriptLibraryString();

  int getFunctionID(const char *name) const;
  WMLScriptValueObject *call(int id, WMLScriptFunctionParameters *parameters);
  const char *getName() const;
  
 private:
  char *buf;
  
  WMLScriptValueObject *funcLength(WMLScriptFunctionParameters *parameters);
  WMLScriptValueObject *funcIsEmpty(WMLScriptFunctionParameters *parameters);
  WMLScriptValueObject *funcCharAt(WMLScriptFunctionParameters *parameters);
  WMLScriptValueObject *funcSubString(WMLScriptFunctionParameters *parameters);
  WMLScriptValueObject *funcFind(WMLScriptFunctionParameters *parameters);
  WMLScriptValueObject *funcReplace(WMLScriptFunctionParameters *parameters);
  WMLScriptValueObject *funcElements(WMLScriptFunctionParameters *parameters);
  WMLScriptValueObject *funcElementAt(WMLScriptFunctionParameters *parameters);
  WMLScriptValueObject *funcRemoveAt(WMLScriptFunctionParameters *parameters);
  WMLScriptValueObject *funcReplaceAt(WMLScriptFunctionParameters *parameters);
  WMLScriptValueObject *funcInsertAt(WMLScriptFunctionParameters *parameters);
  WMLScriptValueObject *funcSqueeze(WMLScriptFunctionParameters *parameters);
  WMLScriptValueObject *funcTrim(WMLScriptFunctionParameters *parameters);
  WMLScriptValueObject *funcCompare(WMLScriptFunctionParameters *parameters);
  WMLScriptValueObject *funcToString(WMLScriptFunctionParameters *parameters);
  WMLScriptValueObject *funcFormat(WMLScriptFunctionParameters *parameters);
  char *substFormat(char *s, int offset, int len, char type, int width, int precision, WMLScriptValueObject *val);
  char *parseFormat(char *p, char *type, int *width, int *precision);
  char *intFormat(char *s, int offset, int len, int width, int precision, WMLScriptValueObject *val);
  char *floatFormat(char *s, int offset, int len, int width, int precision, WMLScriptValueObject *val);
  char *stringFormat(char *s, int offset, int len, int width, int precision, WMLScriptValueObject *val);

};

#endif
