/* 
 * Copyright (c) 2000-2001, catchy.net. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *      - Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 * 
 *      - Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 * 
 *      - All advertising materials mentioning features or use of this software
 *      must display the following acknowledgement: This product includes
 *      software developed by catchy.net and its contributors.
 * 
 *      - Neither name of catchy.net nor the names of its contributors may
 *      be used to endorse or promote products derived from this software
 *      without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY CATCHY.NET AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL CATCHY.NET OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * $Id: WMLScriptParseTreeNodes.h,v 1.5 2001/01/27 14:36:42 wouter Exp $
 */

#ifndef __WMLSCRIPTPARSETREENODES_H__
#define __WMLSCRIPTPARSETREENODES_H__

/**
 * This struct describes the location of a node 
 * in the parse tree in the original source file.
 */
typedef struct {
  /**
   * The line at which this node starts.
   */
  int first_line;
  /**
   * The line at which this node ends.
   */
  int last_line;
  /**
   * The column at which this node starts.
   */
  int first_column;
  /**
   * The column at which this node ends.
   */
  int last_column;
} tokenLoc;

/**
 * Enumerates the types of nodes in the parse tree.
 */
typedef enum { 
  typeCon, 
  typeId, 
  typeOpr 
} nodeEnum;

/**
 * This node type is used for constant values, ie. literals.
 */
typedef struct {
  /**
   * The type of this node (always typeCon).
   */
  nodeEnum type;
  /**
   * The location of this token in the source file.
   */
  tokenLoc location;
  /**
   * The value of this constant.
   */
  struct WMLScriptValueTag value;
} conNodeType;

/**
 * This node type is used for identifiers, e.g. variable and function names.
 */
typedef struct {
  /**
   * The type of this node (always typeId).
   */
  nodeEnum type;
  /**
   * The location of this token in the source file.
   */
  tokenLoc location;
  /**
   * The name of this identifier. The memory allocated for this
   * node adapts according to the length of the identifier.
   */
  char name[1];
} idNodeType;

/**
 * This node type is used for operators.
 */
typedef struct {
  /** 
   * The type of this node (always typeOpr).
   */
  nodeEnum type;
  /**
   * The location of this token in the source file.
   */
  tokenLoc location;
  /**
   * The kind of operator.
   */
  int oper;
  /**
   * The number of operands.
   */
  int nops;
  /**
   * The operands as a array of pointers to other nodes.
   */
  union nodeTypeTag *op[1];

} oprNodeType;

/**
 * This union represents any node type.
 */
typedef union nodeTypeTag {
  /**
   * The type of this node.
   */
  nodeEnum type;
  /**
   * If type is typeCon, then this field is meaningful.
   */
  conNodeType con;
  /**
   * If type is typeId, then this field is meaningful.
   */
  idNodeType id;
  /**
   * If type is typeOpr, then this field is meaningful.
   */
  oprNodeType opr;

} nodeType;

#endif
