/* 
 * Copyright (c) 2000-2001, catchy.net. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *      - Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 * 
 *      - Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 * 
 *      - All advertising materials mentioning features or use of this software
 *      must display the following acknowledgement: This product includes
 *      software developed by catchy.net and its contributors.
 * 
 *      - Neither name of catchy.net nor the names of its contributors may
 *      be used to endorse or promote products derived from this software
 *      without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY CATCHY.NET AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL CATCHY.NET OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * $Id: WMLScriptTester.h,v 1.7 2001/01/27 14:36:42 wouter Exp $
 */

#ifndef __WMLSCRIPTTESTER_H__
#define __WMLSCRIPTTESTER_H__

/**
 * This structure holds a conformance test.
 */
typedef struct TestTag {
  /**
   * The expression to test.
   */
  char *expr;
  /**
   * The expected value.
   */
  WMLScriptValueObject *expected;
} Test;

/**
 * A class for testing the WMLScript module in it's entirety.
 * Prints diagnostics on stdout.
 */
class WMLScriptTester {
 public:
  /**
   * The total number of tests run so far.
   */
  int totalTestsRun;
  /**
   * The total number of tests failed so far.
   */
  int totalTestsFailed;

  WMLScriptTester();
  ~WMLScriptTester();
  int test();
  int testConformance();
  int testErrorHandling();
  int testPragmas();
  
 private:
  WMLScriptLibrary *libLang, *libFloat, *libString, *libURL;

  void runConformanceTests(Test *tests);
  bool errorTestBreak();
  bool errorTestContinue();
  bool errorTestWrongArityLibrary();
  bool errorTestWrongArityFunction();
  bool errorTestWrongArityCall();
  bool errorTestVarDecl();
  bool errorTestVarDeclPost();
  bool errorTestFuncDecl();
  bool errorTestLibFuncDecl();
  bool errorTestLibDecl();
  bool errorTestCallDecl();
  bool errorTestParser();
  bool errorTestAccess();
  bool errorTestLibLinking();
  bool errorTestCallParams();
  bool errorTestCallReturn();
  bool errorTestNestedComments();
  bool errorTestHexNulEscape();
  bool errorTestOctalNulEscape();
  bool errorTestUnicodeNulEscape();
  bool errorTestUnterminatedString();
  bool errorTestUnicodeMultibyteEscape();
  bool testPragma(char *script, 
		  WMLScriptPragmaType expectType,
		  WMLScriptMetaPragmaType expectMetaType,
		  char *expectData1,
		  char *expectData2,
		  char *expectData3);

  WMLScriptValueObject *eval(char *exp);
};

/** testFn points to a member of WMLScriptTester that takes () */
typedef bool (WMLScriptTester::*testFn)();

/** Macro to ease calling member functions by pointer */
#define callMemberFunction(object,ptrToMember)  ((object).*(ptrToMember))

/**
 * This structure holds an error handling test.
 */
typedef struct ErrorTestTag {
  /**
   * Description of test.
   */
  char *description;
  /**
   * Pointer to function which performs test.
   */
  bool (WMLScriptTester::*test) ();
} ErrorTest;

#endif
